<?php

/**
 * @file
 * Definition of Drupal\bunnypoint\BunnypointFormController.
 */

namespace Drupal\bunnypoint;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityFormController;

/**
 * Form controller for the bunnypoint edit/add forms.
 */
class BunnypointFormController extends EntityFormController {

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::form().
   */
  public function form(array $form, array &$form_state, EntityInterface $bunnypoint) {
    $form['label'] = array(
      '#type' => 'textfield',
      '#title' => t('Label'),
      '#maxlength' => 255,
      '#default_value' => $bunnypoint->label(),
      '#description' => t("Example: 'Banner' or 'Highlight'."),
      '#required' => TRUE,
    );
    $form['id'] = array(
      '#type' => 'machine_name',
      '#default_value' => $bunnypoint->id(),
      '#machine_name' => array(
        'exists' => 'bunnypoint_load',
        'source' => array('label'),
      ),
      '#disabled' => (bool) $bunnypoint->id(),
    );

    $form['width'] = array(
      '#type' => 'textfield',
      '#title' => t('Bunnypoint'),
      // http://blog.cloudfour.com/the-ems-have-it-proportional-media-queries-ftw/
      // has some great points for em based responsive layouts, we allow for pixel
      // and em based layouts so long as the same unit is used. We do not explicitly
      // deny separate units, but our orderings assumes the same unit is used.
      '#description' => t('Width point where this bunnypoint is applied. Examples: 960px, 41em. Do not mix pixel and em based widths.'),
      '#default_value' => @$bunnypoint->width,
    );

    $grids = gridbuilder_load_all();
    $grid_list = array();
    foreach ($grids as $grid) {
      $grid_list[$grid->id()] = $grid->label();
    }
    $form['grid'] = array(
      '#type' => 'select',
      '#options' => $grid_list,
      '#default_value' => @$bunnypoint->grid,
      '#description' => t('Grid to be applied to this bunnypoint. <a href="@editgrids">You can edit and add grids</a>.', array('@editgrids' => url('admin/structure/grids'))),
    );

    return parent::form($form, $form_state, $bunnypoint);
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::actions().
   */
  protected function actions(array $form, array &$form_state) {
    // Only includes a Save action for the entity, no direct Delete button.
    return array(
      'submit' => array(
        '#value' => t('Save'),
        '#validate' => array(
          array($this, 'validate'),
        ),
        '#submit' => array(
          array($this, 'submit'),
          array($this, 'save'),
        ),
      ),
    );
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::validate().
   */
  public function validate(array $form, array &$form_state) {
    if (!preg_match('!^\d+(px|em)$!', $form_state['values']['width'])) {
      form_set_error('width', t('Only pixel or em based bunnypoints are supported, for example 960px or 41em.'));
    }
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::save().
   */
  public function save(array $form, array &$form_state) {
    $bunnypoint = $this->getEntity($form_state);
    $bunnypoint->save();

    watchdog('bunnypoint', 'Bunnypoint @label saved.', array('@label' => $bunnypoint->label()), WATCHDOG_NOTICE);
    drupal_set_message(t('Bunnypoint %label saved.', array('%label' => $bunnypoint->label())));

    $form_state['redirect'] = 'admin/structure/bunnypoints';
  }

}

